@@ -3,6 +3,9 @@
 #   Utility function to choose the Kalman filter and smoother
 #######################################################################################################
 MARSSkf = function( MLEobj, only.logLik=FALSE, return.lag.one=TRUE, return.kfas.model=FALSE ) {
+if(is.null(MLEobj$par))
+  stop("MARSSkf: par element of marssMLE object is required.\n")
+#full=TRUE means to return all the kf info, this is currently done by MARSSkfss
 if(MLEobj$fun.kf=="MARSSkfss") 
   return(MARSSkfss(MLEobj))
 if(MLEobj$fun.kf=="MARSSkfas") 
